@@ -3,6 +3,8 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
+from __future__ import annotations
+
 import datetime
 import functools
 import inspect
@@ -15,7 +17,6 @@ import sys
 import threading
 
 from io import StringIO
-from typing import Any, Dict, List, Optional, Set
 
 import pytest
 
@@ -34,6 +35,7 @@ from structlog.processors import (
     JSONRenderer,
     KeyValueRenderer,
     LogfmtRenderer,
+    MaybeTimeStamper,
     StackInfoRenderer,
     TimeStamper,
     UnicodeDecoder,
@@ -46,6 +48,7 @@ from structlog.stdlib import ProcessorFormatter
 from structlog.threadlocal import wrap_dict
 from structlog.typing import EventDict
 from tests.additional_frame import additional_frame
+from tests.utils import CustomError
 
 
 try:
@@ -284,11 +287,9 @@ class TestLogfmtRenderer:
             "invalid key": "somevalue",
         }
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError, match='Invalid key: "invalid key"'):
             LogfmtRenderer()(None, None, event_dict)
 
-        assert 'Invalid key: "invalid key"' == e.value.args[0]
-
 
 class TestJSONRenderer:
     def test_renders_json(self, event_dict):
@@ -365,11 +366,11 @@ class TestTimeStamper:
         A asking for a UNIX timestamp with a timezone that's not UTC raises a
         ValueError.
         """
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(
+            ValueError, match="UNIX timestamps are always UTC."
+        ):
             TimeStamper(utc=False)
 
-        assert "UNIX timestamps are always UTC." == e.value.args[0]
-
     def test_inserts_utc_unix_timestamp_by_default(self):
         """
         Per default a float UNIX timestamp is used.
@@ -440,21 +441,34 @@ class TestTimeStamper:
             None, None, {}
         )
 
-    @pytest.mark.parametrize(
-        ("utc", "expect"),
-        [
-            (True, "1980-03-25T16:00:00Z"),
-            (False, "1980-03-25T17:00:00"),
-        ],
-    )
-    def test_apply_freezegun_after_instantiation(self, utc, expect):
+    def test_apply_freezegun_after_instantiation(self):
         """
-        Instantiate TimeStamper after mocking datetime
+        Freezing time after instantiation of TimeStamper works.
         """
-        ts = TimeStamper(fmt="iso", utc=utc)
+        ts = TimeStamper(fmt="iso", utc=False)
+
         with freeze_time("1980-03-25 16:00:00", tz_offset=1):
             d = ts(None, None, {})
-            assert expect == d["timestamp"]
+
+            assert "1980-03-25T17:00:00" == d["timestamp"]
+
+
+class TestMaybeTimeStamper:
+    def test_overwrite(self):
+        """
+        If there is a timestamp, leave it.
+        """
+        mts = MaybeTimeStamper()
+
+        assert {"timestamp": 42} == mts(None, None, {"timestamp": 42})
+
+    def test_none(self):
+        """
+        If there is no timestamp, add one.
+        """
+        mts = MaybeTimeStamper()
+
+        assert "timestamp" in mts(None, None, {})
 
 
 class TestFormatExcInfo:
@@ -462,13 +476,16 @@ class TestFormatExcInfo:
         """
         The exception formatter can be changed.
         """
+        formatter = ExceptionRenderer(lambda _: "There is no exception!")
+
         try:
-            raise ValueError("test")
-        except ValueError as e:
-            formatter = ExceptionRenderer(lambda _: "There is no exception!")
-            assert formatter(None, None, {"exc_info": e}) == {
-                "exception": "There is no exception!"
-            }
+            raise CustomError("test")
+        except CustomError as e:
+            exc = e
+
+        assert formatter(None, None, {"exc_info": exc}) == {
+            "exception": "There is no exception!"
+        }
 
     @pytest.mark.parametrize("ei", [False, None, ""])
     def test_nop(self, ei):
@@ -509,18 +526,21 @@ class TestFormatExcInfo:
 
     def test_exception(self):
         """
-        Passing exceptions as exc_info is valid on Python 3.
+        Passing exceptions as exc_info is valid.
         """
+        formatter = ExceptionRenderer(lambda exc_info: exc_info)
+
         try:
             raise ValueError("test")
         except ValueError as e:
-            formatter = ExceptionRenderer(lambda exc_info: exc_info)
-            d = formatter(None, None, {"exc_info": e})
-
-            assert {"exception": (ValueError, e, e.__traceback__)} == d
+            exc = e
         else:
             pytest.fail("Exception not raised.")
 
+        assert {
+            "exception": (ValueError, exc, exc.__traceback__)
+        } == formatter(None, None, {"exc_info": exc})
+
     def test_exception_without_traceback(self):
         """
         If an Exception is missing a traceback, render it anyway.
@@ -540,7 +560,17 @@ class TestFormatExcInfo:
         except ValueError as e:
             a = format_exc_info(None, None, {"exc_info": e})
             b = ExceptionRenderer()(None, None, {"exc_info": e})
-            assert a == b
+
+        assert a == b
+
+    @pytest.mark.parametrize("ei", [True, (None, None, None)])
+    def test_no_exception(self, ei):
+        """
+        A missing exception does not blow up.
+        """
+        assert {"exception": "MISSING"} == format_exc_info(
+            None, None, {"exc_info": ei}
+        )
 
 
 class TestUnicodeEncoder:
@@ -700,7 +730,7 @@ class TestExceptionPrettyPrinter:
         assert "XXX" in sio.getvalue()
 
 
-@pytest.fixture
+@pytest.fixture()
 def sir():
     return StackInfoRenderer()
 
@@ -723,6 +753,9 @@ class TestStackInfoRenderer:
         assert "stack" in ed
 
     def test_renders_correct_stack(self, sir):
+        """
+        The rendered stack is correct.
+        """
         ed = sir(None, None, {"stack_info": True})
 
         assert 'ed = sir(None, None, {"stack_info": True})' in ed["stack"]
@@ -790,43 +823,37 @@ class TestCallsiteParameterAdder:
         }
         assert self.parameter_strings == self.get_callsite_parameters().keys()
 
-    @pytest.mark.xfail(
-        reason=(
-            "CallsiteParameterAdder cannot "
-            "determine the callsite for async calls."
-        )
-    )
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_async(self) -> None:
         """
         Callsite information for async invocations are correct.
         """
-        try:
-            string_io = StringIO()
+        string_io = StringIO()
 
-            class StingIOLogger(structlog.PrintLogger):
-                def __init__(self):
-                    super().__init__(file=string_io)
+        class StingIOLogger(structlog.PrintLogger):
+            def __init__(self):
+                super().__init__(file=string_io)
 
-            processor = self.make_processor(None, ["concurrent", "threading"])
-            structlog.configure(
-                processors=[processor, JSONRenderer()],
-                logger_factory=StingIOLogger,
-                wrapper_class=structlog.stdlib.AsyncBoundLogger,
-                cache_logger_on_first_use=True,
-            )
+        processor = self.make_processor(None, ["concurrent", "threading"])
+        structlog.configure(
+            processors=[processor, JSONRenderer()],
+            logger_factory=StingIOLogger,
+            wrapper_class=structlog.stdlib.AsyncBoundLogger,
+            cache_logger_on_first_use=True,
+        )
 
-            logger = structlog.stdlib.get_logger()
+        logger = structlog.stdlib.get_logger()
 
-            callsite_params = self.get_callsite_parameters()
-            await logger.info("baz")
+        callsite_params = self.get_callsite_parameters()
+        await logger.info("baz")
+        logger_params = json.loads(string_io.getvalue())
 
-            assert {"event": "baz", **callsite_params} == json.loads(
-                string_io.getvalue()
-            )
+        # These are different when running under async
+        for key in ["thread", "thread_name"]:
+            callsite_params.pop(key)
+            logger_params.pop(key)
 
-        finally:
-            structlog.reset_defaults()
+        assert {"event": "baz", **callsite_params} == logger_params
 
     def test_additional_ignores(self, monkeypatch: pytest.MonkeyPatch) -> None:
         """
@@ -838,28 +865,17 @@ class TestCallsiteParameterAdder:
         processor = self.make_processor(None, additional_ignores)
         event_dict: EventDict = {"event": test_message}
 
-        # `functools.partial` is used instead of a lambda because a lambda will
-        # add an additional frame in a module that should not be ignored.
-        _sys_getframe = functools.partial(additional_frame, sys._getframe)
-
-        # WARNING: The below three lines are sensitive to relative line numbers
-        # (i.e. the invocation of processor must be two lines after the
-        # invocation of get_callsite_parameters) and is order sensitive (i.e.
-        # monkeypatch.setattr must occur after get_callsite_parameters but
-        # before invocation of processor).
-        callsite_params = self.get_callsite_parameters(2)
-        monkeypatch.setattr(sys, "_getframe", value=_sys_getframe)
+        # Warning: the next two lines must appear exactly like this to make
+        # line numbers match.
+        callsite_params = self.get_callsite_parameters(1)
         actual = processor(None, None, event_dict)
 
-        expected = {
-            "event": test_message,
-            **callsite_params,
-        }
+        expected = {"event": test_message, **callsite_params}
 
         assert expected == actual
 
     @pytest.mark.parametrize(
-        "origin, parameter_strings",
+        ("origin", "parameter_strings"),
         itertools.product(
             ["logging", "structlog"],
             [
@@ -875,7 +891,7 @@ class TestCallsiteParameterAdder:
     def test_processor(
         self,
         origin: str,
-        parameter_strings: Optional[Set[str]],
+        parameter_strings: set[str] | None,
     ):
         """
         The correct callsite parameters are added to event dictionaries.
@@ -914,15 +930,12 @@ class TestCallsiteParameterAdder:
         callsite_params = self.filter_parameter_dict(
             callsite_params, parameter_strings
         )
-        expected = {
-            "event": test_message,
-            **callsite_params,
-        }
+        expected = {"event": test_message, **callsite_params}
 
         assert expected == actual
 
     @pytest.mark.parametrize(
-        "setup, origin, parameter_strings",
+        ("setup", "origin", "parameter_strings"),
         itertools.product(
             ["common-without-pre", "common-with-pre", "shared", "everywhere"],
             ["logging", "structlog"],
@@ -940,7 +953,7 @@ class TestCallsiteParameterAdder:
         self,
         setup: str,
         origin: str,
-        parameter_strings: Optional[Set[str]],
+        parameter_strings: set[str] | None,
     ) -> None:
         """
         Logging output contains the correct callsite parameters.
@@ -983,7 +996,7 @@ class TestCallsiteParameterAdder:
             callsite_params = self.get_callsite_parameters()
             logger.info(test_message)
         elif origin == "structlog":
-            ctx: Dict[str, Any] = {}
+            ctx = {}
             bound_logger = BoundLogger(
                 logger,
                 [*common_processors, ProcessorFormatter.wrap_for_formatter],
@@ -1002,55 +1015,54 @@ class TestCallsiteParameterAdder:
             for key, value in json.loads(string_io.getvalue()).items()
             if not key.startswith("_")
         }
-        expected = {
-            "event": test_message,
-            **callsite_params,
-        }
+        expected = {"event": test_message, **callsite_params}
 
         assert expected == actual
 
     @classmethod
     def make_processor(
         cls,
-        parameter_strings: Optional[Set[str]],
-        additional_ignores: Optional[List[str]] = None,
+        parameter_strings: set[str] | None,
+        additional_ignores: list[str] | None = None,
     ) -> CallsiteParameterAdder:
         """
         Creates a ``CallsiteParameterAdder`` with parameters matching the
-        supplied ``parameter_strings`` values and with the supplied
-        ``additional_ignores`` values.
+        supplied *parameter_strings* values and with the supplied
+        *additional_ignores* values.
 
-        :param parameter_strings:
-            Strings for which corresponding ``CallsiteParameters`` should be
-            included in the resulting ``CallsiteParameterAdded``.
-        :param additional_ignores:
-            Used as ``additional_ignores`` for the resulting
-            ``CallsiteParameterAdded``.
+        Args:
+            parameter_strings:
+                Strings for which corresponding ``CallsiteParameters`` should
+                be included in the resulting ``CallsiteParameterAdded``.
+
+            additional_ignores:
+                Used as *additional_ignores* for the resulting
+                ``CallsiteParameterAdded``.
         """
         if parameter_strings is None:
             return CallsiteParameterAdder(
                 additional_ignores=additional_ignores
             )
-        else:
-            parameters = cls.filter_parameters(parameter_strings)
-            return CallsiteParameterAdder(
-                parameters=parameters,
-                additional_ignores=additional_ignores,
-            )
+
+        parameters = cls.filter_parameters(parameter_strings)
+        return CallsiteParameterAdder(
+            parameters=parameters,
+            additional_ignores=additional_ignores,
+        )
 
     @classmethod
     def filter_parameters(
-        cls, parameter_strings: Optional[Set[str]]
-    ) -> Set[CallsiteParameter]:
+        cls, parameter_strings: set[str] | None
+    ) -> set[CallsiteParameter]:
         """
         Returns a set containing all ``CallsiteParameter`` members with values
         that are in ``parameter_strings``.
 
-        :param parameter_strings:
-            The parameters strings for which corresponding
-            ``CallsiteParameter`` members should be
-            returned. If this value is `None` then all
-            ``CallsiteParameter`` will be returned.
+        Args:
+            parameter_strings:
+                The parameters strings for which corresponding
+                ``CallsiteParameter`` members should be returned. If this value
+                is `None` then all ``CallsiteParameter`` will be returned.
         """
         if parameter_strings is None:
             return cls._all_parameters
@@ -1062,15 +1074,16 @@ class TestCallsiteParameterAdder:
 
     @classmethod
     def filter_parameter_dict(
-        cls, input: Dict[str, Any], parameter_strings: Optional[Set[str]]
-    ) -> Dict[str, Any]:
+        cls, input: dict[str, object], parameter_strings: set[str] | None
+    ) -> dict[str, object]:
         """
-        Returns a dictionary that is equivalent to ``input`` but with all keys
-        not in ``parameter_strings`` removed.
+        Returns a dictionary that is equivalent to *input* but with all keys
+        not in *parameter_strings* removed.
 
-        :param parameter_strings:
-            The keys to keep in the dictionary, if this value is ``None`` then
-            all keys matching ``cls.parameter_strings`` are kept.
+        Args:
+            parameter_strings:
+                The keys to keep in the dictionary, if this value is ``None``
+                then all keys matching ``cls.parameter_strings`` are kept.
         """
         if parameter_strings is None:
             parameter_strings = cls.parameter_strings
@@ -1081,14 +1094,15 @@ class TestCallsiteParameterAdder:
         }
 
     @classmethod
-    def get_callsite_parameters(cls, offset: int = 1) -> Dict[str, Any]:
+    def get_callsite_parameters(cls, offset: int = 1) -> dict[str, object]:
         """
         This function creates dictionary of callsite parameters for the line
         that is ``offset`` lines after the invocation of this function.
 
-        :param offset:
-            The amount of lines after the invocation of this function that
-            callsite parameters should be generated for.
+        Args:
+            offset:
+                The amount of lines after the invocation of this function that
+                callsite parameters should be generated for.
         """
         frame_info = inspect.stack()[1]
         frame_traceback = inspect.getframeinfo(frame_info[0])
