@@ -3,6 +3,8 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
+from __future__ import annotations
+
 import json
 import logging
 import logging.config
@@ -10,7 +12,7 @@ import os
 import sys
 
 from io import StringIO
-from typing import Any, Callable, Collection, Dict, Optional, Set
+from typing import Any, Callable, Collection, Dict
 
 import pytest
 import pytest_asyncio
@@ -22,10 +24,9 @@ from structlog import (
     ReturnLogger,
     configure,
     get_context,
-    reset_defaults,
 )
 from structlog._config import _CONFIG
-from structlog._log_levels import _NAME_TO_LEVEL, CRITICAL, WARN
+from structlog._log_levels import CRITICAL, NAME_TO_LEVEL, WARN
 from structlog.dev import ConsoleRenderer
 from structlog.exceptions import DropEvent
 from structlog.processors import JSONRenderer, KeyValueRenderer
@@ -108,7 +109,11 @@ class TestLoggerFactory:
         )
 
     def test_deduces_correct_caller(self):
+        """
+        It will find the correct caller.
+        """
         logger = _FixedFindCallerLogger("test")
+
         file_name, line_number, func_name = logger.findCaller()[:3]
 
         assert file_name == os.path.realpath(__file__)
@@ -124,21 +129,25 @@ class TestLoggerFactory:
         assert "testing, is_, fun" in stack_info
 
     def test_no_stack_info_by_default(self):
+        """
+        If we don't ask for stack_info, it won't be returned.
+        """
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller()
 
         assert None is stack_info
 
-    def test_find_caller(self, monkeypatch):
+    def test_find_caller(self, caplog):
+        """
+        The caller is found.
+        """
         logger = LoggerFactory()()
-        log_handle = call_recorder(lambda x: None)
-        monkeypatch.setattr(logger, "handle", log_handle)
-        logger.error("Test")
-        log_record = log_handle.calls[0].args[0]
 
-        assert log_record.funcName == "test_find_caller"
-        assert log_record.name == __name__
-        assert log_record.filename == os.path.basename(__file__)
+        logger.error("Test")
+
+        assert caplog.text.startswith(
+            "ERROR    tests.test_stdlib:test_stdlib.py"
+        )
 
     def test_sets_correct_logger(self):
         """
@@ -184,7 +193,8 @@ class TestFilterByLevel:
 
 class TestBoundLogger:
     @pytest.mark.parametrize(
-        ("method_name"), ["debug", "info", "warning", "error", "critical"]
+        ("method_name"),
+        ["debug", "info", "warning", "error", "exception", "critical"],
     )
     def test_proxies_to_correct_method(self, method_name):
         """
@@ -194,14 +204,6 @@ class TestBoundLogger:
 
         assert method_name == getattr(bl, method_name)("event")
 
-    def test_proxies_exception(self):
-        """
-        BoundLogger.exception is proxied to Logger.error.
-        """
-        bl = BoundLogger(ReturnLogger(), [return_method_name], {})
-
-        assert "error" == bl.exception("event")
-
     def test_proxies_log(self):
         """
         BoundLogger.exception.log() is proxied to the appropriate method.
@@ -237,7 +239,7 @@ class TestBoundLogger:
         assert bound_logger_attribute == stdlib_logger_attribute
 
     @pytest.mark.parametrize(
-        "method_name,method_args",
+        ("method_name", "method_args"),
         [
             ("addHandler", [None]),
             ("removeHandler", [None]),
@@ -467,7 +469,7 @@ class TestPositionalArgumentsFormatter:
 
 
 class TestAddLogLevelNumber:
-    @pytest.mark.parametrize("level, number", _NAME_TO_LEVEL.items())
+    @pytest.mark.parametrize(("level", "number"), NAME_TO_LEVEL.items())
     def test_log_level_number_added(self, level, number):
         """
         The log level number is added to the event dict.
@@ -486,13 +488,16 @@ class TestAddLogLevel:
 
         assert "error" == event_dict["level"]
 
-    def test_log_level_alias_normalized(self):
+    @pytest.mark.parametrize(
+        ("alias", "normalized"), [("warn", "warning"), ("exception", "error")]
+    )
+    def test_log_level_alias_normalized(self, alias, normalized):
         """
         The normalized name of the log level is added to the event dict.
         """
-        event_dict = add_log_level(None, "warn", {})
+        event_dict = add_log_level(None, alias, {})
 
-        assert "warning" == event_dict["level"]
+        assert normalized == event_dict["level"]
 
 
 @pytest.fixture(name="make_log_record")
@@ -539,7 +544,7 @@ class TestAddLoggerName:
         assert name == event_dict["logger"]
 
 
-def extra_dict() -> Dict[str, Any]:
+def extra_dict() -> dict[str, Any]:
     """
     A dict to be passed in the `extra` parameter of the `logging` module's log
     methods.
@@ -559,11 +564,11 @@ def extra_dict_fixture():
 
 class TestExtraAdder:
     @pytest.mark.parametrize(
-        "allow, misses",
+        ("allow", "misses"),
         [
             (None, None),
             ({}, None),
-            *[({key}, None) for key in extra_dict().keys()],
+            *[({key}, None) for key in extra_dict()],
             ({"missing"}, {"missing"}),
             ({"missing", "keys"}, {"missing"}),
             ({"this", "x_int"}, None),
@@ -572,9 +577,9 @@ class TestExtraAdder:
     def test_add_extra(
         self,
         make_log_record: Callable[[], logging.LogRecord],
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
-        misses: Optional[Set[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
+        misses: set[str] | None,
     ):
         """
         Extra attributes of a LogRecord object are added to the event dict.
@@ -601,11 +606,11 @@ class TestExtraAdder:
         assert {} == actual
 
     @pytest.mark.parametrize(
-        "allow, misses",
+        ("allow", "misses"),
         [
             (None, None),
             ({}, None),
-            *[({key}, None) for key in extra_dict().keys()],
+            *[({key}, None) for key in extra_dict()],
             ({"missing"}, {"missing"}),
             ({"missing", "keys"}, {"missing"}),
             ({"this", "x_int"}, None),
@@ -613,9 +618,9 @@ class TestExtraAdder:
     )
     def test_add_extra_e2e(
         self,
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
-        misses: Optional[Set[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
+        misses: set[str] | None,
     ):
         """
         Values passed in the `extra` parameter of the `logging` module's log
@@ -652,20 +657,18 @@ class TestExtraAdder:
     def _copy_allowed(
         cls,
         event_dict: EventDict,
-        extra_dict: Dict[str, Any],
-        allow: Optional[Collection[str]],
+        extra_dict: dict[str, Any],
+        allow: Collection[str] | None,
     ) -> EventDict:
         if allow is None:
             return {**event_dict, **extra_dict}
-        else:
-            return {
-                **event_dict,
-                **{
-                    key: value
-                    for key, value in extra_dict.items()
-                    if key in allow
-                },
-            }
+
+        return {
+            **event_dict,
+            **{
+                key: value for key, value in extra_dict.items() if key in allow
+            },
+        }
 
 
 class TestRenderToLogKW:
@@ -720,7 +723,8 @@ def _configure_for_processor_formatter():
     """
     Configure structlog to use ProcessorFormatter.
 
-    Reset both structlog and logging setting after the test.
+    Reset logging setting after the test (structlog is reset automatically
+    before all tests).
     """
     configure(
         processors=[add_log_level, ProcessorFormatter.wrap_for_formatter],
@@ -731,14 +735,13 @@ def _configure_for_processor_formatter():
     yield
 
     logging.basicConfig()
-    reset_defaults()
 
 
 def configure_logging(
     pre_chain,
     logger=None,
     pass_foreign_args=False,
-    renderer=ConsoleRenderer(colors=False),
+    renderer=ConsoleRenderer(colors=False),  # noqa: B008
 ):
     """
     Configure logging to use ProcessorFormatter.
@@ -831,7 +834,7 @@ class TestProcessorFormatter:
         If `pass_foreign_args` is `True` we set the `positional_args` key in
         the `event_dict` before clearing args.
         """
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging((test_processor,), pass_foreign_args=True)
 
         positional_args = {"foo": "bar"}
@@ -879,7 +882,7 @@ class TestProcessorFormatter:
 
         assert (
             "",
-            "foo                            [sample-name]  [in test_foreign_pr"
+            "foo                            [sample-name] [in test_foreign_pr"
             "e_chain_add_logger_name]\n",
         ) == capsys.readouterr()
 
@@ -910,7 +913,7 @@ class TestProcessorFormatter:
         If non-structlog record contains exc_info, foreign_pre_chain functions
         have access to it.
         """
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging((test_processor,), renderer=KeyValueRenderer())
 
         try:
@@ -929,26 +932,26 @@ class TestProcessorFormatter:
         ProcessorFormatter should not have changed it.
         """
 
-        class MyException(Exception):
+        class MyError(Exception):
             pass
 
         def add_excinfo(logger, log_method, event_dict):
             event_dict["exc_info"] = sys.exc_info()
             return event_dict
 
-        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        test_processor = call_recorder(lambda _, __, event_dict: event_dict)
         configure_logging(
             (add_excinfo, test_processor), renderer=KeyValueRenderer()
         )
 
         try:
-            raise MyException("oh no")
+            raise MyError("oh no")
         except Exception:
             logging.getLogger().error("okay")
 
         event_dict = test_processor.calls[0].args[2]
 
-        assert MyException is event_dict["exc_info"][0]
+        assert MyError is event_dict["exc_info"][0]
 
     def test_other_handlers_get_original_record(self):
         """
@@ -1116,16 +1119,113 @@ class TestProcessorFormatter:
             {"foo": "bar", "_record": "foo", "_from_structlog": True},
         )
 
+    def test_non_string_message_warning(self):
+        """
+        A warning is raised if the last processor in
+        ProcessorFormatter.processors doesn't return a string.
+        """
+        configure_logging(None)
+        logger = logging.getLogger()
+
+        formatter = ProcessorFormatter(
+            processors=[lambda *args, **kwargs: {"foo": "bar"}],
+        )
+        logger.handlers[0].setFormatter(formatter)
+
+        with pytest.warns(
+            RuntimeWarning,
+            match="The last processor in ProcessorFormatter.processors must return a string",
+        ):
+            logger.info("baz")
+
+    def test_logrecord_exc_info(self):
+        """
+        LogRecord.exc_info is set consistently for structlog and non-structlog
+        log records.
+        """
+        configure_logging(None)
+
+        # This doesn't test ProcessorFormatter itself directly, but it's
+        # relevant to setups where ProcessorFormatter is used, i.e. where
+        # handlers will receive LogRecord objects that come from both structlog
+        # and non-structlog loggers.
+
+        records: Dict[  # noqa: UP006 - dict isn't generic until Python 3.9
+            str, logging.LogRecord
+        ] = {}
+
+        class DummyHandler(logging.Handler):
+            def emit(self, record):
+                # Don't do anything; just store the record in the records dict
+                # by its message, so we can assert things about it.
+                if isinstance(record.msg, dict):
+                    records[record.msg["event"]] = record
+                else:
+                    records[record.msg] = record
+
+        stdlib_logger = logging.getLogger()
+        structlog_logger = get_logger()
+
+        # It doesn't matter which logger we add the handler to here.
+        stdlib_logger.addHandler(DummyHandler())
+
+        try:
+            raise Exception("foo")
+        except Exception:
+            stdlib_logger.exception("bar")
+            structlog_logger.exception("baz")
+
+        stdlib_record = records.pop("bar")
+
+        assert "bar" == stdlib_record.msg
+        assert stdlib_record.exc_info
+        assert Exception is stdlib_record.exc_info[0]
+        assert ("foo",) == stdlib_record.exc_info[1].args
+
+        structlog_record = records.pop("baz")
+
+        assert "baz" == structlog_record.msg["event"]
+        assert True is structlog_record.msg["exc_info"]
+        assert structlog_record.exc_info
+        assert Exception is structlog_record.exc_info[0]
+        assert ("foo",) == structlog_record.exc_info[1].args
+
+        assert not records
+
+    def test_use_get_message_false(self):
+        """
+        If use_get_message_is False, the event is obtained using
+        str(record.msg) instead of calling record.getMessage. That means
+        positional formatting is not performed.
+        """
+        event_dicts = []
+
+        def capture(_, __, ed):
+            event_dicts.append(ed.copy())
+
+            return str(ed)
+
+        proc = ProcessorFormatter(processors=[capture], use_get_message=False)
+
+        record = logging.LogRecord(
+            "foo",
+            logging.INFO,
+            "path.py",
+            42,
+            "le msg: %s",
+            ("keep separate",),
+            None,
+        )
+
+        assert proc.format(record)
+        assert "le msg: %s" == event_dicts[0]["event"]
+
 
 @pytest_asyncio.fixture(name="abl")
 async def _abl(cl):
     return AsyncBoundLogger(cl, context={}, processors=[])
 
 
-@pytest.mark.skipif(
-    sys.version_info[:2] < (3, 7),
-    reason="AsyncBoundLogger is only for Python 3.7 and later.",
-)
 class TestAsyncBoundLogger:
     def test_sync_bl(self, abl, cl):
         """
@@ -1137,31 +1237,28 @@ class TestAsyncBoundLogger:
             CapturedCall(method_name="info", args=(), kwargs={"event": "test"})
         ] == cl.calls
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_protocol(self, abl):
         """
         AsyncBoundLogger is a proper BindableLogger.
         """
         assert isinstance(abl, BindableLogger)
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_correct_levels(self, abl, cl, stdlib_log_method):
         """
         The proxy methods call the correct upstream methods.
         """
         await getattr(abl.bind(foo="bar"), stdlib_log_method)("42")
 
-        aliases = {"exception": "error", "warn": "warning"}
+        aliases = {"warn": "warning"}
 
         alias = aliases.get(stdlib_log_method)
-        if alias:
-            expect = alias
-        else:
-            expect = stdlib_log_method
+        expect = alias if alias else stdlib_log_method
 
         assert expect == cl.calls[0].method_name
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_log_method(self, abl, cl):
         """
         The `log` method is proxied too.
@@ -1170,7 +1267,7 @@ class TestAsyncBoundLogger:
 
         assert "error" == cl.calls[0].method_name
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_exception(self, abl, cl):
         """
         `exception` makes sure 'exc_info" is set, if it's not set already.
@@ -1185,7 +1282,7 @@ class TestAsyncBoundLogger:
         assert ValueError is ei[0]
         assert ("omg",) == ei[1].args
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_exception_do_not_overwrite(self, abl, cl):
         """
         `exception` leaves exc_info be, if it's set.
@@ -1202,7 +1299,7 @@ class TestAsyncBoundLogger:
         ei = cl.calls[0].kwargs["exc_info"]
         assert (o1, o2, o3) == ei
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_bind_unbind(self, cl):
         """
         new/bind/unbind/try_unbind are correctly propagated.
@@ -1233,7 +1330,7 @@ class TestAsyncBoundLogger:
         assert {} == l5._context
         assert l4 is not l5
 
-    @pytest.mark.asyncio
+    @pytest.mark.asyncio()
     async def test_integration(self, capsys):
         """
         Configure and log an actual entry.
@@ -1257,16 +1354,12 @@ class TestAsyncBoundLogger:
             "level": "info",
         } == json.loads(capsys.readouterr().out)
 
-        reset_defaults()
-
 
 @pytest.mark.parametrize("log_level", [None, 45])
 def test_recreate_defaults(log_level):
     """
     Recreate defaults configures structlog and -- if asked -- logging.
     """
-    reset_defaults()
-
     logging.basicConfig(
         stream=sys.stderr,
         level=1,
@@ -1279,11 +1372,6 @@ def test_recreate_defaults(log_level):
     assert dict is _CONFIG.default_context_class
     assert isinstance(_CONFIG.logger_factory, LoggerFactory)
 
-    # 3.7 doesn't have the force keyword and we don't care enough to
-    # re-implement it.
-    if sys.version_info < (3, 8):
-        return
-
     log = get_logger().bind()
     if log_level is not None:
         assert log_level == log.getEffectiveLevel()
